#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

import base64
import yaml
from jinja2 import Environment

def createEnvironment() -> Environment:
    env = Environment(extensions=['jinja2.ext.loopcontrols'])
    env.filters['to_nice_yaml'] = to_nice_yaml
    env.filters['kv_comma_pairs_to_json'] = kv_comma_pairs_to_json
    env.filters['to_base64'] = to_base64
    return env
    

# Jinja filter
def to_nice_yaml(value, default_flow_style=False, sort_keys=False, indent=2):
    return yaml.dump(value, default_flow_style=False, sort_keys=sort_keys, indent=indent)

# Jinja filter
def kv_comma_pairs_to_json(value):
    obj = {}
    for kvpair in value.split(","):
        if kvpair and kvpair != '' and '=' in kvpair:
            key = kvpair.split("=")[0]
            val = kvpair.split("=")[1]
            obj[key] = val

    return json.dumps(obj)

# Jinja filter
def to_base64(value):
    return base64.b64encode(value.encode('utf-8')).decode('utf-8')





#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

CONFIG_DATA_KEY = "cicdstatemgr"
SECRET_DATA_KEY = "cicdstatemgr"
STATE = 'state'
CONFIG_DATA = 'configData'
SECRET_DATA = 'secretData'
CICD_CONTEXT_DATA_ID = 'cicdContextDataId'
CICD_CONTEXT_NAME = 'cicdContextName'

class CicdContextData():
    cicdContextData:dict

    def __init__(self,cicdContextData:dict):
        self.cicdContextData = cicdContextData


    def getCicdContextDataId(self) -> str:
        return self.cicdContextData[STATE][CICD_CONTEXT_DATA_ID]


    def getCicdContextData(self) -> str:
        return self.cicdContextData


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

import yaml
import os
import redis
import logging

from . import CicdContextDataSource

class DataSource(CicdContextDataSource):
    host = "localhost"
    port = 6379
    db = 0
    conn = None
    password = None
    username = 'default'
    isPrimary = False

    def __init__(self, config:dict):
        self.host =  config['host'] if 'host' in config else None
        self.port = config['port'] if 'port' in config else None
        self.db = config['db'] if 'db' in config else None
        self.password = config['password'] if 'password' in config else None
        self.username = config['username'] if 'username' in config else None

        if 'isPrimary' in config:
            self.isPrimary = config['isPrimary']

        self.init()

        logging.debug("{} -> {} isPrimary:{} isLocal:{}".format(self.get_name(),self.host,self.isPrimary,self.is_local()))

    def is_local(self) -> bool:
        return False

    def get_name(self) -> str:
        return 'redis'

    def is_primary(self) -> bool:
        return self.isPrimary

    def init(self):
        self.conn = redis.Redis(host=self.host, port=self.port, db=self.db, username=self.username, password=self.password)

    def persist(self, cicdContextDataId:str, cicdContextData:object):
        obj = yaml.dump(cicdContextData, default_flow_style=False, sort_keys=False)
        self.conn.set(cicdContextDataId, obj)

    def load(self, cicdContextDataId:str) -> dict:
        self.init()
        yamlStr = self.conn.get(cicdContextDataId)
        return yaml.load(yamlStr, Loader=yaml.FullLoader)

    def supports_load(self) -> bool:
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

import abc
import logging
import importlib

from .. import CicdContextData

class CicdContextDataSource(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, subclass):
        return (hasattr(subclass, 'persist') and 
                callable(subclass.persist) and 
                hasattr(subclass, 'load') and 
                callable(subclass.load) or 
                NotImplemented)

    @abc.abstractmethod
    def get_name(self) -> str:
        raise NotImplementedError

    @abc.abstractmethod
    def is_primary(self) -> bool:
        raise NotImplementedError

    @abc.abstractmethod
    def is_local(self) -> bool:
        raise NotImplementedError

    @abc.abstractmethod
    def persist(self,  cicdContextDataId:str, cicdContextData:object):
        raise NotImplementedError

    @abc.abstractmethod
    def load(self, cicdContextDataId:str) -> dict:
        raise NotImplementedError

    @abc.abstractmethod
    def supports_load(self) -> bool:
        raise NotImplementedError


class DataSourceMgr():

    dataSources:dict = {}
    primaryDataSource:CicdContextDataSource = None

    def __init__(self, dsConfigs:dict):
        for dsName in dsConfigs:
            logging.debug("DataSourceMgr() Initializing datasource: {}".format(dsName))

            try:
                module = importlib.import_module('.{}'.format(dsName), package=__name__)
                ds = module.DataSource(dsConfigs[dsName])
                self.dataSources[dsName] = ds
                if ds.is_primary():
                    if self.primaryDataSource:
                        logging.error("DataSourceMgr() you cannot have more than one isPrimary data source defined: {} , current primary = {}".format(dsName,self.primaryDataSource.get_name()))
                    else:
                        logging.error("DataSourceMgr() primary ds = {}".format(dsName))
                        self.primaryDataSource = ds

            except ModuleNotFoundError as e:
                logging.error("No datasource module found by name: {} .. ignoring".format(dsName))

        if not self.primaryDataSource:
            raise Exception("DataSourceMgr() you have no isPrimary=True datasource configured! Invalid")

    def persist(self, cicdContextData:CicdContextData, skipPrimary=False):
        logging.debug("DataSourceMgr.persist() skipPrimary={} cicdContextDataId={}".format(skipPrimary,cicdContextData.getCicdContextDataId()))

        for dsName in self.dataSources:
            ds = self.dataSources[dsName]
            if skipPrimary and ds.is_primary():
                logging.debug("DataSourceMgr.persist() skipping primary: {} as skipPrimary={}".format(dsName,skipPrimary))
                continue

            logging.debug("DataSourceMgr.persist() persisting in: {}".format(dsName))
            ds.persist(cicdContextData.getCicdContextDataId(),cicdContextData.getCicdContextData())

    def load(self, cicdContextDataId:str, fromPrimary=True, fromLocal=False):
        logging.debug("DataSourceMgr.load() fromPrimary={} fromLocal={} cicdContextDataId={}".format(fromPrimary,fromLocal,cicdContextDataId))

        if fromPrimary and self.primaryDataSource.is_primary():

            if fromLocal and not self.primaryDataSource.is_local():
                raise Exception("DataSourceMgr.load() cannot pass " + \
                    "fromPrimary={} and fromLocal={} as the primaryDataSource[{}] is_local()=false".format(fromPrimary,fromLocal,self.primaryDataSource.get_name()))

            return self.primaryDataSource.load(cicdContextDataId)
            
        else:
            for dsName in self.dataSources:
                ds = self.dataSources[dsName]
                if not ds.is_primary() and ds.supports_load():
                    if fromLocal and ds.is_local():
                        logging.debug("DataSourceMgr.load() loading from {}".format(dsName))
                        return ds.load(cicdContextDataId)

        raise Exception("DataSourceMgr.load() cannot pass " + \
            "fromPrimary={} and fromLocal={}, no datasources meet this criteria".format(fromPrimary,fromLocal))

